
.flap-background {
	background-color: @theme_bg_color;
}

.header-box {
	min-height: 36px;
}

.header-subtitle {
	font-size: small;
}

.profile-key {
	font-size: xx-small;
}

.account-entry {
	padding: 4px 20px;
}

.contact-entry {
	padding: 2px;
}

.chat-entry {
	padding: 4px;
}

.message-avatar {
	margin: 4px;
}

.message-box {
	border-radius: 12px;
	padding: 8px 12px;
	margin: 4px;
	font-size: small;
}

.message-box:not(.sent) {
	background-color: mix(@theme_bg_color, @theme_fg_color, 0.1);
	color: @theme_fg_color;
	margin-right: 20px;
}

.message-box.sent {
	background-color: mix(@theme_bg_color, @theme_selected_bg_color, 0.8);
	color: @theme_selected_fg_color;
}

.message-box.sent, .message-box.status {
	margin-left: 20px;
}

.message-content:not(.status) {
	padding-top: 4px;
}

.message-box.status > .message-content {
	padding: 2px 4px;
}

.timestamp-label {
	font-size: x-small;
}

.picker-switcher-box {
	padding: 0px 8px;
}

.emoji-flow-box {
	font-size: large;
}

.settings-entry, .details-entry {
	margin: 8px;
}

.details-group {
	margin-top: 16px;
}

.details-group-title {
	margin-left: 8px;
	margin-right: 8px;
	margin-bottom: 4px;
}

